#!/usr/bin/perl -w

# impgraph -- mailgraph-like IMP login/failed status
#
# based on mailgraph by David Schweikert <dws@ee.ethz.ch>
# released under the GNU General Public License

use strict;

use RRDs;
use File::Tail 0.8;
use Getopt::Long;
use Parse::Syslog;

my $VERSION = 0.2;

# config
my $rrdstep = 60; # secs
my $cdpstep = 5; # 5 PDP => 1 CDP

my $rrd = '/var/db/impgraph/impgraph.rrd';
my $this_minute;
my %sum = ( login => 0, failed => 0);
my $rrd_inited = 0;

my $logfile = '/var/log/horde.log';

my %opt = ();

# prototypes
sub usage();
sub main();
sub process_line($);
sub init_rrd($);
sub event($$);
sub update($);

sub usage() {
	print <<EOF;

	$0 [-l logfile] [-y year] [-v|verbose]
	
EOF
	exit;
}

sub main() {
	GetOptions(\%opt, 'help|h', 'logfile|l=s', 'year|y=i', 'verbose|h');
	usage if $opt{help};

	$logfile = $opt{logfile} if defined($opt{logfile});

	my $file = File::Tail->new(name => $logfile, tail => -1);
	my $parser = Parse::Syslog->new($file, year => $opt{year}, arrayref => 1);

	while (my $sl = $parser->next) {
		process_line($sl);
	}
}

sub process_line($) {
	my $sl = shift;
	my $time = $sl->[0];
	my $prog = $sl->[2];
	my $text = $sl->[4];

	# [imp] Login success for jschen [65.95.45.18] to {ccms.ntu.edu.tw:993} [on line 136 of "/home/services/www/data/horde/imp/redirect.php"]
	# Relogin
	#[imp] FAILED LOGIN 219.68.40.96 to ms86.ntu.edu.tw:995[pop3/ssl/novalidate-cert] as b3401111 [on line 234 of "/home/services/www/data/horde/imp/lib/Auth/imp.php"]
	
	if ($prog eq "HORDE") {
		$text =~ /^\[(\w+)\] /;
		my $prog = $1;

		if ($prog eq "imp") {
			if ($text =~ /Login success for/) {
				$text =~ s/.+ Login success for (\w+) \[(\d+\.\d+\.\d+\.\d+)\] to \{([\w\.]+):\d+\} .+/$1 $2 $3/;
				#print "LOGIN $time $text\n";
				event($time, 'login');
			} elsif ($text =~ /Relogin success for/) {
				$text =~ s/.+ Relogin success for (\w+) \[(\d+\.\d+\.\d+\.\d+)\] to \{([\w\.]+):\d+\} .+/$1 $2 $3/;
				#print "RELOGIN $time $text\n";
				event($time, 'login');
			} elsif ($text =~ /FAILED LOGIN/) {
				$text =~ s/.+ FAILED LOGIN (\d+\.\d+\.\d+\.\d+) to ([\w\.]+):\d+.* as (\w+) .+/$3 $1 $2/;
				#print "FAILED $time $text\n";
				event($time, 'failed');
			}
		}
	}
}

sub init_rrd($) {
	my $m = shift;
	my $rows = int(3600*24 / ($rrdstep*$cdpstep));
	my $realrows = int($rows*1.1); # ensure that the full range is covered
	my $day_steps = $cdpstep;
	# use multiples, otherwise rrdtool could choose the wrong RRA
	my $week_steps = $day_steps*7;
	my $month_steps = $week_steps*5;
	my $year_steps = $month_steps*12;
	if(not -f $rrd) {
		RRDs::create($rrd, '--start', $m, '--step', $rrdstep,
				'DS:login:ABSOLUTE:'.($rrdstep*2).':0:U',
				'DS:failed:ABSOLUTE:'.($rrdstep*2).':0:U',
				"RRA:AVERAGE:0.5:$day_steps:$realrows",   # day
				"RRA:AVERAGE:0.5:$week_steps:$realrows",  # week
				"RRA:AVERAGE:0.5:$month_steps:$realrows", # month
				"RRA:AVERAGE:0.5:$year_steps:$realrows",  # year
				"RRA:MAX:0.5:$day_steps:$realrows",   # day
				"RRA:MAX:0.5:$week_steps:$realrows",  # week
				"RRA:MAX:0.5:$month_steps:$realrows", # month
				"RRA:MAX:0.5:$year_steps:$realrows",  # year
				);
		$this_minute = $m;
	}
	else {
		$this_minute = RRDs::last($rrd) + $rrdstep;
	}

	$rrd_inited=1;
}

sub event($$) {
	my ($t, $type) = @_;
	update($t) and $sum{$type}++;
}

# returns 1 if $sum should be updated
sub update($) {
	my $t = shift;
	my $m = $t - $t%$rrdstep;
	init_rrd($m) unless $rrd_inited;
	return 1 if $m == $this_minute;
	return 0 if $m < $this_minute;

	print "update $this_minute:$sum{login}:$sum{failed}\n" if $opt{verbose};
	RRDs::update $rrd, "$this_minute:$sum{login}:$sum{failed}";
	if($m > $this_minute+$rrdstep) {
		for(my $sm=$this_minute+$rrdstep;$sm<$m;$sm+=$rrdstep) {
			print "update $sm:0:0 (SKIP)\n" if $opt{verbose};
			RRDs::update $rrd, "$sm:0:0";
		}
	}
	$this_minute = $m;
	$sum{login}=0;
	$sum{failed}=0;
	return 1;
}

&main;
